import pytest
from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse

from leaflet_storage.models import Map

from .base import MapFactory

pytestmark = pytest.mark.django_db


def test_anonymous_can_edit_if_status_anonymous(map):
    anonymous = AnonymousUser()
    map.edit_status = map.ANONYMOUS
    map.save()
    assert map.can_edit(anonymous)


def test_anonymous_cannot_edit_if_not_status_anonymous(map):
    anonymous = AnonymousUser()
    map.edit_status = map.OWNER
    map.save()
    assert not map.can_edit(anonymous)


def test_non_editors_can_edit_if_status_anonymous(map, user):
    assert map.owner != user
    map.edit_status = map.ANONYMOUS
    map.save()
    assert map.can_edit(user)


def test_non_editors_cannot_edit_if_not_status_anonymous(map, user):
    map.edit_status = map.OWNER
    map.save()
    assert not map.can_edit(user)


def test_editors_cannot_edit_if_status_owner(map, user):
    map.edit_status = map.OWNER
    map.editors.add(user)
    map.save()
    assert not map.can_edit(user)


def test_editors_can_edit_if_status_editors(map, user):
    map.edit_status = map.EDITORS
    map.editors.add(user)
    map.save()
    assert map.can_edit(user)


def test_logged_in_user_should_be_allowed_for_anonymous_map_with_anonymous_edit_status(map, user, rf):  # noqa
    map.owner = None
    map.edit_status = map.ANONYMOUS
    map.save()
    url = reverse('map_update', kwargs={'map_id': map.pk})
    request = rf.get(url)
    request.user = user
    assert map.can_edit(user, request)


def test_clone_should_return_new_instance(map, user):
    clone = map.clone()
    assert map.pk != clone.pk
    assert u"Clone of " + map.name == clone.name
    assert map.settings == clone.settings
    assert map.center == clone.center
    assert map.zoom == clone.zoom
    assert map.licence == clone.licence
    assert map.tilelayer == clone.tilelayer


def test_clone_should_keep_editors(map, user):
    map.editors.add(user)
    clone = map.clone()
    assert map.pk != clone.pk
    assert user in map.editors.all()
    assert user in clone.editors.all()


def test_clone_should_update_owner_if_passed(map, user):
    clone = map.clone(owner=user)
    assert map.pk != clone.pk
    assert map.owner != clone.owner
    assert user == clone.owner


def test_clone_should_clone_datalayers_and_features_too(map, user, datalayer):
    clone = map.clone()
    assert map.pk != clone.pk
    assert map.datalayer_set.count() == 1
    assert clone.datalayer_set.count() == 1
    other = clone.datalayer_set.all()[0]
    assert datalayer in map.datalayer_set.all()
    assert other.pk != datalayer.pk
    assert other.name == datalayer.name
    assert other.geojson is not None
    assert other.geojson.path != datalayer.geojson.path


def test_publicmanager_should_get_only_public_maps(map, user, licence):
    map.share_status = map.PUBLIC
    open_map = MapFactory(owner=user, licence=licence, share_status=Map.OPEN)
    private_map = MapFactory(owner=user, licence=licence,
                             share_status=Map.PRIVATE)
    assert map in Map.public.all()
    assert open_map not in Map.public.all()
    assert private_map not in Map.public.all()



import shutil
import tempfile

import pytest
from django.core.signing import get_cookie_signer

from .base import (DataLayerFactory, LicenceFactory, MapFactory,
                   TileLayerFactory, UserFactory)

TMP_ROOT = tempfile.mkdtemp()


def pytest_configure(config):
    from django.conf import settings
    settings.MEDIA_ROOT = TMP_ROOT


def pytest_unconfigure(config):
    shutil.rmtree(TMP_ROOT, ignore_errors=True)


@pytest.fixture
def user():
    return UserFactory(password="123123")


@pytest.fixture
def licence():
    return LicenceFactory()


@pytest.fixture
def map(licence, tilelayer):
    user = UserFactory(username="Gabriel", password="123123")
    return MapFactory(owner=user, licence=licence)


@pytest.fixture
def anonymap(map):
    map.owner = None
    map.save()
    return map


@pytest.fixture
def cookieclient(client, anonymap):
    key, value = anonymap.signed_cookie_elements
    client.cookies[key] = get_cookie_signer(salt=key).sign(value)
    return client


@pytest.fixture
def allow_anonymous(settings):
    settings.LEAFLET_STORAGE_ALLOW_ANONYMOUS = True


@pytest.fixture
def datalayer(map):
    return DataLayerFactory(map=map, name="Default Datalayer")


@pytest.fixture
def tilelayer():
    return TileLayerFactory()



import pytest

from leaflet_storage.models import Map

pytestmark = pytest.mark.django_db


def test_can_use_dict(map):
    d = {'locateControl': True}
    map.settings = d
    map.save()
    assert Map.objects.get(pk=map.pk).settings == d


def test_can_set_item(map):
    d = {'locateControl': True}
    map.settings = d
    map.save()
    map_inst = Map.objects.get(pk=map.pk)
    map_inst.settings['color'] = 'DarkGreen'
    assert map_inst.settings['locateControl'] is True


def test_should_return_a_dict_if_none(map):
    map.settings = None
    map.save()
    assert Map.objects.get(pk=map.pk).settings == {}


def test_should_not_double_dumps(map):
    map.settings = '{"locate": true}'
    map.save()
    assert Map.objects.get(pk=map.pk).settings == {'locate': True}



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-20 11:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leaflet_storage', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='datalayer',
            name='rank',
            field=models.SmallIntegerField(default=0),
        ),
    ]



import json

import pytest
from django.core.files.base import ContentFile
from django.core.urlresolvers import reverse

from leaflet_storage.models import DataLayer, Map

from .base import MapFactory

pytestmark = pytest.mark.django_db


@pytest.fixture
def post_data():
    return {
        "name": 'name',
        "display_on_load": True,
        "rank": 0,
        "geojson": '{"type":"FeatureCollection","features":[{"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-3.1640625,53.014783245859235],[-3.1640625,51.86292391360244],[-0.50537109375,51.385495069223204],[1.16455078125,52.38901106223456],[-0.41748046875,53.91728101547621],[-2.109375,53.85252660044951],[-3.1640625,53.014783245859235]]]},"properties":{"_storage_options":{},"name":"Ho god, sounds like a polygouine"}},{"type":"Feature","geometry":{"type":"LineString","coordinates":[[1.8017578124999998,51.16556659836182],[-0.48339843749999994,49.710272582105695],[-3.1640625,50.0923932109388],[-5.60302734375,51.998410382390325]]},"properties":{"_storage_options":{},"name":"Light line"}},{"type":"Feature","geometry":{"type":"Point","coordinates":[0.63720703125,51.15178610143037]},"properties":{"_storage_options":{},"name":"marker he"}}],"_storage":{"displayOnLoad":true,"name":"new name","id":1668,"remoteData":{},"color":"LightSeaGreen","description":"test"}}'  # noqa
    }


def test_get(client, settings, datalayer):
    url = reverse('datalayer_view', args=(datalayer.pk, ))
    response = client.get(url)
    if getattr(settings, 'LEAFLET_STORAGE_XSENDFILE_HEADER', None):
        assert response['ETag'] is not None
    assert response['Last-Modified'] is not None
    assert response['Cache-Control'] is not None
    assert 'Content-Encoding' not in response
    j = json.loads(response.content.decode())
    assert '_storage' in j
    assert 'features' in j
    assert j['type'] == 'FeatureCollection'


def test_update(client, datalayer, map, post_data):
    url = reverse('datalayer_update', args=(map.pk, datalayer.pk))
    client.login(username=map.owner.username, password="123123")
    name = 'new name'
    rank = 2
    post_data['name'] = name
    post_data['rank'] = rank
    response = client.post(url, post_data, follow=True)
    assert response.status_code == 200
    modified_datalayer = DataLayer.objects.get(pk=datalayer.pk)
    assert modified_datalayer.name == name
    assert modified_datalayer.rank == rank
    # Test response is a json
    j = json.loads(response.content.decode())
    assert "id" in j
    assert datalayer.pk == j['id']


def test_should_not_be_possible_to_update_with_wrong_map_id_in_url(client, datalayer, map, post_data):  # noqa
    other_map = MapFactory(owner=map.owner)
    url = reverse('datalayer_update', args=(other_map.pk, datalayer.pk))
    client.login(username=map.owner.username, password="123123")
    name = 'new name'
    post_data['name'] = name
    response = client.post(url, post_data, follow=True)
    assert response.status_code == 403
    modified_datalayer = DataLayer.objects.get(pk=datalayer.pk)
    assert modified_datalayer.name == datalayer.name


def test_delete(client, datalayer, map):
    url = reverse('datalayer_delete', args=(map.pk, datalayer.pk))
    client.login(username=map.owner.username, password='123123')
    response = client.post(url, {}, follow=True)
    assert response.status_code == 200
    assert not DataLayer.objects.filter(pk=datalayer.pk).count()
    # Check that map has not been impacted
    assert Map.objects.filter(pk=map.pk).exists()
    # Test response is a json
    j = json.loads(response.content.decode())
    assert 'info' in j


def test_should_not_be_possible_to_delete_with_wrong_map_id_in_url(client, datalayer, map):  # noqa
    other_map = MapFactory(owner=map.owner)
    url = reverse('datalayer_delete', args=(other_map.pk, datalayer.pk))
    client.login(username=map.owner.username, password='123123')
    response = client.post(url, {}, follow=True)
    assert response.status_code == 403
    assert DataLayer.objects.filter(pk=datalayer.pk).exists()


def test_get_gzipped(client, datalayer, settings):
    url = reverse('datalayer_view', args=(datalayer.pk, ))
    response = client.get(url, HTTP_ACCEPT_ENCODING='gzip')
    if getattr(settings, 'LEAFLET_STORAGE_XSENDFILE_HEADER', None):
        assert response['ETag'] is not None
    assert response['Last-Modified'] is not None
    assert response['Cache-Control'] is not None
    assert response['Content-Encoding'] == 'gzip'


def test_optimistic_concurrency_control_with_good_etag(client, datalayer, map, post_data):  # noqa
    # Get Etag
    url = reverse('datalayer_view', args=(datalayer.pk, ))
    response = client.get(url)
    etag = response['ETag']
    url = reverse('datalayer_update',
                  args=(map.pk, datalayer.pk))
    client.login(username=map.owner.username, password="123123")
    name = 'new name'
    post_data['name'] = 'new name'
    response = client.post(url, post_data, follow=True, HTTP_IF_MATCH=etag)
    assert response.status_code == 200
    modified_datalayer = DataLayer.objects.get(pk=datalayer.pk)
    assert modified_datalayer.name == name


def test_optimistic_concurrency_control_with_bad_etag(client, datalayer, map, post_data):  # noqa
    url = reverse('datalayer_update', args=(map.pk, datalayer.pk))
    client.login(username=map.owner.username, password='123123')
    name = 'new name'
    post_data['name'] = name
    response = client.post(url, post_data, follow=True, HTTP_IF_MATCH='xxx')
    assert response.status_code == 412
    modified_datalayer = DataLayer.objects.get(pk=datalayer.pk)
    assert modified_datalayer.name != name


def test_optimistic_concurrency_control_with_empty_etag(client, datalayer, map, post_data):  # noqa
    url = reverse('datalayer_update', args=(map.pk, datalayer.pk))
    client.login(username=map.owner.username, password='123123')
    name = 'new name'
    post_data['name'] = name
    response = client.post(url, post_data, follow=True, HTTP_IF_MATCH=None)
    assert response.status_code == 200
    modified_datalayer = DataLayer.objects.get(pk=datalayer.pk)
    assert modified_datalayer.name == name


def test_versions_should_return_versions(client, datalayer, map, settings):
    root = datalayer.storage_root()
    datalayer.geojson.storage.save(
        '%s/%s_1440924889.geojson' % (root, datalayer.pk),
        ContentFile("{}"))
    datalayer.geojson.storage.save(
        '%s/%s_1440923687.geojson' % (root, datalayer.pk),
        ContentFile("{}"))
    datalayer.geojson.storage.save(
        '%s/%s_1440918637.geojson' % (root, datalayer.pk),
        ContentFile("{}"))
    url = reverse('datalayer_versions', args=(datalayer.pk, ))
    versions = json.loads(client.get(url).content.decode())
    assert len(versions['versions']) == 4
    version = {'name': '%s_1440918637.geojson' % datalayer.pk, 'size': 2,
               'at': '1440918637'}
    assert version in versions['versions']


def test_version_should_return_one_version_geojson(client, datalayer, map):
    root = datalayer.storage_root()
    name = '%s_1440924889.geojson' % datalayer.pk
    datalayer.geojson.storage.save('%s/%s' % (root, name), ContentFile("{}"))
    url = reverse('datalayer_version', args=(datalayer.pk, name))
    assert client.get(url).content.decode() == "{}"



 # -*- coding:utf-8 -*-

import json

import pytest
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse

from leaflet_storage.models import DataLayer, Map

from .base import login_required

pytestmark = pytest.mark.django_db
User = get_user_model()


@pytest.fixture
def post_data():
    return {
        'name': 'name',
        'center': '{"type":"Point","coordinates":[13.447265624999998,48.94415123418794]}',  # noqa
        'settings': '{"type":"Feature","geometry":{"type":"Point","coordinates":[5.0592041015625,52.05924589011585]},"properties":{"tilelayer":{"maxZoom":20,"url_template":"http://{s}.tile.openstreetmap.fr/hot/{z}/{x}/{y}.png","minZoom":0,"attribution":"HOT and friends"},"licence":"","description":"","name":"test enrhûmé","tilelayersControl":true,"displayDataBrowserOnLoad":false,"displayPopupFooter":true,"displayCaptionOnLoad":false,"miniMap":true,"moreControl":true,"scaleControl":true,"zoomControl":true,"datalayersControl":true,"zoom":8}}'  # noqa
    }


def test_create(client, user, post_data):
    url = reverse('map_create')
    # POST only mendatory fields
    name = 'test-map-with-new-name'
    post_data['name'] = name
    client.login(username=user.username, password="123123")
    response = client.post(url, post_data)
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    created_map = Map.objects.latest('pk')
    assert j['id'] == created_map.pk
    assert created_map.name == name


def test_map_create_permissions(client):
    url = reverse('map_create')
    # POST anonymous
    response = client.post(url, {})
    assert login_required(response)


def test_map_update_access(client, map, user):
    url = reverse('map_update', kwargs={'map_id': map.pk})
    # GET anonymous
    response = client.get(url)
    assert login_required(response)
    # POST anonymous
    response = client.post(url, {})
    assert login_required(response)
    # GET with wrong permissions
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert response.status_code == 403
    # POST with wrong permissions
    client.login(username=user.username, password="123123")
    response = client.post(url, {})
    assert response.status_code == 403


def test_map_update_permissions_access(client, map, user):
    url = reverse('map_update_permissions', kwargs={'map_id': map.pk})
    # GET anonymous
    response = client.get(url)
    assert login_required(response)
    # POST anonymous
    response = client.post(url, {})
    assert login_required(response)
    # GET with wrong permissions
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert response.status_code == 403
    # POST with wrong permissions
    client.login(username=user.username, password="123123")
    response = client.post(url, {})
    assert response.status_code == 403


def test_update(client, map, post_data):
    url = reverse('map_update', kwargs={'map_id': map.pk})
    # POST only mendatory fields
    name = 'new map name'
    post_data['name'] = name
    client.login(username=map.owner.username, password="123123")
    response = client.post(url, post_data)
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    assert 'html' not in j
    updated_map = Map.objects.get(pk=map.pk)
    assert j['id'] == updated_map.pk
    assert updated_map.name == name


def test_delete(client, map, datalayer):
    url = reverse('map_delete', args=(map.pk, ))
    client.login(username=map.owner.username, password="123123")
    response = client.post(url, {}, follow=True)
    assert response.status_code == 200
    assert not Map.objects.filter(pk=map.pk).exists()
    assert not DataLayer.objects.filter(pk=datalayer.pk).exists()
    # Check that user has not been impacted
    assert User.objects.filter(pk=map.owner.pk).exists()
    # Test response is a json
    j = json.loads(response.content.decode())
    assert 'redirect' in j


def test_wrong_slug_should_redirect_to_canonical(client, map):
    url = reverse('map', kwargs={'pk': map.pk, 'slug': 'wrong-slug'})
    canonical = reverse('map', kwargs={'pk': map.pk,
                                       'slug': map.slug})
    response = client.get(url)
    assert response.status_code == 301
    assert response['Location'] == canonical


def test_wrong_slug_should_redirect_with_query_string(client, map):
    url = reverse('map', kwargs={'pk': map.pk, 'slug': 'wrong-slug'})
    url = "{}?allowEdit=0".format(url)
    canonical = reverse('map', kwargs={'pk': map.pk,
                                       'slug': map.slug})
    canonical = "{}?allowEdit=0".format(canonical)
    response = client.get(url)
    assert response.status_code == 301
    assert response['Location'] == canonical


def test_should_not_consider_the_query_string_for_canonical_check(client, map):
    url = reverse('map', kwargs={'pk': map.pk, 'slug': map.slug})
    url = "{}?allowEdit=0".format(url)
    response = client.get(url)
    assert response.status_code == 200


def test_short_url_should_redirect_to_canonical(client, map):
    url = reverse('map_short_url', kwargs={'pk': map.pk})
    canonical = reverse('map', kwargs={'pk': map.pk,
                                       'slug': map.slug})
    response = client.get(url)
    assert response.status_code == 301
    assert response['Location'] == canonical


def test_old_url_should_redirect_to_canonical(client, map):
    url = reverse(
        'map_old_url',
        kwargs={'username': map.owner.username, 'slug': map.slug}
    )
    canonical = reverse('map', kwargs={'pk': map.pk,
                                       'slug': map.slug})
    response = client.get(url)
    assert response.status_code == 301
    assert response['Location'] == canonical


def test_clone_map_should_create_a_new_instance(client, map):
    assert Map.objects.count() == 1
    url = reverse('map_clone', kwargs={'map_id': map.pk})
    client.login(username=map.owner.username, password="123123")
    response = client.post(url)
    assert response.status_code == 200
    assert Map.objects.count() == 2
    clone = Map.objects.latest('pk')
    assert clone.pk != map.pk
    assert clone.name == u"Clone of " + map.name


def test_user_not_allowed_should_not_clone_map(client, map, user):
    assert Map.objects.count() == 1
    url = reverse('map_clone', kwargs={'map_id': map.pk})
    map.edit_status = map.OWNER
    map.save()
    response = client.post(url)
    assert login_required(response)
    client.login(username=user.username, password="123123")
    response = client.post(url)
    assert response.status_code == 403
    map.edit_status = map.ANONYMOUS
    map.save()
    client.logout()
    response = client.post(url)
    assert response.status_code == 403
    assert Map.objects.count() == 1


def test_clone_should_set_cloner_as_owner(client, map, user):
    url = reverse('map_clone', kwargs={'map_id': map.pk})
    map.edit_status = map.EDITORS
    map.editors.add(user)
    map.save()
    client.login(username=user.username, password="123123")
    response = client.post(url)
    assert response.status_code == 200
    assert Map.objects.count() == 2
    clone = Map.objects.latest('pk')
    assert clone.pk != map.pk
    assert clone.name == u"Clone of " + map.name
    assert clone.owner == user


def test_map_creation_should_allow_unicode_names(client, map, post_data):
    url = reverse('map_create')
    # POST only mendatory fields
    name = u'Академический'
    post_data['name'] = name
    client.login(username=map.owner.username, password="123123")
    response = client.post(url, post_data)
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    created_map = Map.objects.latest('pk')
    assert j['id'] == created_map.pk
    assert created_map.name == name
    # Lower case of the russian original name
    # self.assertEqual(created_map.slug, u"академический")
    # for now we fallback to "map", see unicode_name branch
    assert created_map.slug == 'map'


def test_anonymous_can_access_map_with_share_status_public(client, map):
    url = reverse('map', args=(map.slug, map.pk))
    map.share_status = map.PUBLIC
    map.save()
    response = client.get(url)
    assert response.status_code == 200


def test_anonymous_can_access_map_with_share_status_open(client, map):
    url = reverse('map', args=(map.slug, map.pk))
    map.share_status = map.OPEN
    map.save()
    response = client.get(url)
    assert response.status_code == 200


def test_anonymous_cannot_access_map_with_share_status_private(client, map):
    url = reverse('map', args=(map.slug, map.pk))
    map.share_status = map.PRIVATE
    map.save()
    response = client.get(url)
    assert response.status_code == 403


def test_owner_can_access_map_with_share_status_private(client, map):
    url = reverse('map', args=(map.slug, map.pk))
    map.share_status = map.PRIVATE
    map.save()
    client.login(username=map.owner.username, password="123123")
    response = client.get(url)
    assert response.status_code == 200


def test_editors_can_access_map_with_share_status_private(client, map, user):
    url = reverse('map', args=(map.slug, map.pk))
    map.share_status = map.PRIVATE
    map.editors.add(user)
    map.save()
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert response.status_code == 200


def test_non_editor_cannot_access_map_if_share_status_private(client, map, user):  # noqa
    url = reverse('map', args=(map.slug, map.pk))
    map.share_status = map.PRIVATE
    map.save()
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert response.status_code == 403


def test_map_geojson_view(client, map):
    url = reverse('map_geojson', args=(map.pk, ))
    response = client.get(url)
    j = json.loads(response.content.decode())
    assert 'type' in j


def test_only_owner_can_delete(client, map, user):
    map.editors.add(user)
    url = reverse('map_delete', kwargs={'map_id': map.pk})
    client.login(username=user.username, password="123123")
    response = client.post(url, {}, follow=True)
    assert response.status_code == 403


def test_map_editors_do_not_see_owner_change_input(client, map, user):
    map.editors.add(user)
    map.edit_status = map.EDITORS
    map.save()
    url = reverse('map_update_permissions', kwargs={'map_id': map.pk})
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert 'id_owner' not in response


def test_logged_in_user_can_edit_map_editable_by_anonymous(client, map, user):
    map.owner = None
    map.edit_status = map.ANONYMOUS
    map.save()
    client.login(username=user.username, password="123123")
    url = reverse('map_update', kwargs={'map_id': map.pk})
    new_name = 'this is my new name'
    data = {
        'center': '{"type":"Point","coordinates":[13.447265624999998,48.94415123418794]}',  # noqa
        'name': new_name
    }
    response = client.post(url, data)
    assert response.status_code == 200
    assert Map.objects.get(pk=map.pk).name == new_name


@pytest.mark.usefixtures('allow_anonymous')
def test_anonymous_create(cookieclient, post_data):
    url = reverse('map_create')
    # POST only mendatory fields
    name = 'test-map-with-new-name'
    post_data['name'] = name
    response = cookieclient.post(url, post_data)
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    created_map = Map.objects.latest('pk')
    assert j['id'] == created_map.pk
    assert created_map.name == name
    key, value = created_map.signed_cookie_elements
    assert key in cookieclient.cookies


@pytest.mark.usefixtures('allow_anonymous')
def test_anonymous_update_without_cookie_fails(client, anonymap, post_data):  # noqa
    url = reverse('map_update', kwargs={'map_id': anonymap.pk})
    response = client.post(url, post_data)
    assert response.status_code == 403


@pytest.mark.usefixtures('allow_anonymous')
def test_anonymous_update_with_cookie_should_work(cookieclient, anonymap, post_data):  # noqa
    url = reverse('map_update', kwargs={'map_id': anonymap.pk})
    # POST only mendatory fields
    name = 'new map name'
    post_data['name'] = name
    response = cookieclient.post(url, post_data)
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    updated_map = Map.objects.get(pk=anonymap.pk)
    assert j['id'] == updated_map.pk


@pytest.mark.usefixtures('allow_anonymous')
def test_anonymous_delete(cookieclient, anonymap):
    url = reverse('map_delete', args=(anonymap.pk, ))
    response = cookieclient.post(url, {}, follow=True)
    assert response.status_code == 200
    assert not Map.objects.filter(pk=anonymap.pk).count()
    # Test response is a json
    j = json.loads(response.content.decode())
    assert 'redirect' in j


@pytest.mark.usefixtures('allow_anonymous')
def test_no_cookie_cant_delete(client, anonymap):
    url = reverse('map_delete', args=(anonymap.pk, ))
    response = client.post(url, {}, follow=True)
    assert response.status_code == 403


@pytest.mark.usefixtures('allow_anonymous')
def test_anonymous_edit_url(cookieclient, anonymap):
    url = anonymap.get_anonymous_edit_url()
    canonical = reverse('map', kwargs={'pk': anonymap.pk,
                                       'slug': anonymap.slug})
    response = cookieclient.get(url)
    assert response.status_code == 302
    assert response['Location'] == canonical
    key, value = anonymap.signed_cookie_elements
    assert key in cookieclient.cookies


@pytest.mark.usefixtures('allow_anonymous')
def test_bad_anonymous_edit_url_should_return_403(cookieclient, anonymap):
    url = anonymap.get_anonymous_edit_url()
    url = reverse(
        'map_anonymous_edit_url',
        kwargs={'signature': "%s:badsignature" % anonymap.pk}
    )
    response = cookieclient.get(url)
    assert response.status_code == 403


@pytest.mark.usefixtures('allow_anonymous')
def test_authenticated_user_with_cookie_is_attached_as_owner(cookieclient, anonymap, post_data, user):  # noqa
    url = reverse('map_update', kwargs={'map_id': anonymap.pk})
    cookieclient.login(username=user.username, password="123123")
    assert anonymap.owner is None
    # POST only mendatory filds
    name = 'new map name for authenticat_anonymoused user'
    post_data['name'] = name
    response = cookieclient.post(url, post_data)
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    updated_map = Map.objects.get(pk=anonymap.pk)
    assert j['id'] == updated_map.pk
    assert updated_map.owner.pk, user.pk


@pytest.mark.usefixtures('allow_anonymous')
def test_clone_anonymous_map_should_not_be_possible_if_user_is_not_allowed(client, anonymap, user):  # noqa
    assert Map.objects.count() == 1
    url = reverse('map_clone', kwargs={'map_id': anonymap.pk})
    anonymap.edit_status = anonymap.OWNER
    anonymap.save()
    response = client.post(url)
    assert response.status_code == 403
    client.login(username=user.username, password="123123")
    response = client.post(url)
    assert response.status_code == 403
    assert Map.objects.count() == 1


@pytest.mark.usefixtures('allow_anonymous')
def test_clone_map_should_be_possible_if_edit_status_is_anonymous(client, anonymap):  # noqa
    assert Map.objects.count() == 1
    url = reverse('map_clone', kwargs={'map_id': anonymap.pk})
    anonymap.edit_status = anonymap.ANONYMOUS
    anonymap.save()
    response = client.post(url)
    assert response.status_code == 200
    assert Map.objects.count() == 2
    clone = Map.objects.latest('pk')
    assert clone.pk != anonymap.pk
    assert clone.name == 'Clone of ' + anonymap.name
    assert clone.owner is None


@pytest.mark.usefixtures('allow_anonymous')
def test_anyone_can_access_anonymous_map(cookieclient, anonymap):
    url = reverse('map', args=(anonymap.slug, anonymap.pk))
    anonymap.share_status = anonymap.PUBLIC
    response = cookieclient.get(url)
    assert response.status_code == 200
    anonymap.share_status = anonymap.OPEN
    response = cookieclient.get(url)
    assert response.status_code == 200
    anonymap.share_status = anonymap.PRIVATE
    response = cookieclient.get(url)
    assert response.status_code == 200



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-10 06:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leaflet_storage', '0002_auto_20160520_1114'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='datalayer',
            options={'ordering': ('rank',)},
        ),
        migrations.AlterField(
            model_name='pictogram',
            name='pictogram',
            field=models.ImageField(upload_to='pictogram'),
        ),
    ]



import json

import factory
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse

from leaflet_storage.forms import DEFAULT_CENTER
from leaflet_storage.models import DataLayer, Licence, Map, TileLayer

User = get_user_model()


class LicenceFactory(factory.DjangoModelFactory):
    name = "WTFPL"

    class Meta:
        model = Licence


class TileLayerFactory(factory.DjangoModelFactory):
    name = "Test zoom layer"
    url_template = "http://{s}.test.org/{z}/{x}/{y}.png"
    attribution = "Test layer attribution"

    class Meta:
        model = TileLayer


class UserFactory(factory.DjangoModelFactory):
    username = 'Joe'
    email = factory.LazyAttribute(
        lambda a: '{0}@example.com'.format(a.username).lower())

    @classmethod
    def _prepare(cls, create, **kwargs):
        password = kwargs.pop('password', None)
        user = super(UserFactory, cls)._prepare(create, **kwargs)
        if password:
            user.set_password(password)
            if create:
                user.save()
        return user

    class Meta:
        model = User


class MapFactory(factory.DjangoModelFactory):
    name = "test map"
    slug = "test-map"
    center = DEFAULT_CENTER
    settings = {
        'geometry': {
            'coordinates': [13.447265624999998, 48.94415123418794],
            'type': 'Point'
        },
        'properties': {
            'datalayersControl': True,
            'description': 'Which is just the Danube, at the end',
            'displayCaptionOnLoad': False,
            'displayDataBrowserOnLoad': False,
            'displayPopupFooter': False,
            'licence': '',
            'miniMap': False,
            'moreControl': True,
            'name': 'Cruising on the Donau',
            'scaleControl': True,
            'tilelayer': {
                'attribution': u'\xa9 OSM Contributors',
                'maxZoom': 18,
                'minZoom': 0,
                'url_template': 'http://{s}.osm.fr/{z}/{x}/{y}.png'
            },
            'tilelayersControl': True,
            'zoom': 7,
            'zoomControl': True
        },
        'type': 'Feature'
    }

    licence = factory.SubFactory(LicenceFactory)
    owner = factory.SubFactory(UserFactory)

    class Meta:
        model = Map


class DataLayerFactory(factory.DjangoModelFactory):
    map = factory.SubFactory(MapFactory)
    name = "test datalayer"
    description = "test description"
    display_on_load = True
    geojson = factory.django.FileField(data="""{"type":"FeatureCollection","features":[{"type":"Feature","geometry":{"type":"Point","coordinates":[13.68896484375,48.55297816440071]},"properties":{"_storage_options":{"color":"DarkCyan","iconClass":"Ball"},"name":"Here","description":"Da place anonymous again 755"}}],"_storage":{"displayOnLoad":true,"name":"Donau","id":926}}""")  # noqa

    class Meta:
        model = DataLayer


def login_required(response):
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    assert 'login_required' in j
    redirect_url = reverse('login')
    assert j['login_required'] == redirect_url
    return True



from django.conf.urls import url
from django.contrib.auth.views import login
from django.views.decorators.csrf import ensure_csrf_cookie
from django.views.decorators.cache import never_cache, cache_control

from . import views
from .decorators import jsonize_view, map_permissions_check,\
    login_required_if_not_anonymous_allowed
from .utils import decorated_patterns

urlpatterns = [
    url(r'^login/$', jsonize_view(login), name='login'),  # noqa
    url(r'^login/popup/end/$', views.LoginPopupEnd.as_view(),
        name='login_popup_end'),
    url(r'^logout/$', views.logout, name='logout'),
    url(r'^map/(?P<pk>\d+)/geojson/$', views.MapViewGeoJSON.as_view(),
        name='map_geojson'),
    url(r'^map/(?P<username>[-_\w]+)/(?P<slug>[-_\w]+)/$',
        views.MapOldUrl.as_view(), name='map_old_url'),
    url(r'^map/anonymous-edit/(?P<signature>.+)$',
        views.MapAnonymousEditUrl.as_view(), name='map_anonymous_edit_url'),
    url(r'^m/(?P<pk>\d+)/$', views.MapShortUrl.as_view(),
        name='map_short_url'),
    url(r'^pictogram/json/$', views.PictogramJSONList.as_view(),
        name='pictogram_list_json'),
]
urlpatterns += decorated_patterns(cache_control(must_revalidate=True),
    url(r'^datalayer/(?P<pk>[\d]+)/$', views.DataLayerView.as_view(), name='datalayer_view'),  # noqa
    url(r'^datalayer/(?P<pk>[\d]+)/versions/$', views.DataLayerVersions.as_view(), name='datalayer_versions'),  # noqa
    url(r'^datalayer/(?P<pk>[\d]+)/(?P<name>[_\w]+.geojson)$', views.DataLayerVersion.as_view(), name='datalayer_version'),  # noqa
)
urlpatterns += decorated_patterns([ensure_csrf_cookie],
    url(r'^map/(?P<slug>[-_\w]+)_(?P<pk>\d+)$', views.MapView.as_view(), name='map'),  # noqa
    url(r'^map/new/$', views.MapNew.as_view(), name='map_new'),
)
urlpatterns += decorated_patterns(
    [login_required_if_not_anonymous_allowed, never_cache],
    url(r'^map/create/$', views.MapCreate.as_view(), name='map_create'),
)
urlpatterns += decorated_patterns(
    [map_permissions_check, never_cache],
    url(r'^map/(?P<map_id>[\d]+)/update/settings/$', views.MapUpdate.as_view(),
        name='map_update'),
    url(r'^map/(?P<map_id>[\d]+)/update/permissions/$',
        views.UpdateMapPermissions.as_view(), name='map_update_permissions'),
    url(r'^map/(?P<map_id>[\d]+)/update/delete/$',
        views.MapDelete.as_view(), name='map_delete'),
    url(r'^map/(?P<map_id>[\d]+)/update/clone/$',
        views.MapClone.as_view(), name='map_clone'),
    url(r'^map/(?P<map_id>[\d]+)/datalayer/create/$',
        views.DataLayerCreate.as_view(), name='datalayer_create'),
    url(r'^map/(?P<map_id>[\d]+)/datalayer/update/(?P<pk>\d+)/$',
        views.DataLayerUpdate.as_view(), name='datalayer_update'),
    url(r'^map/(?P<map_id>[\d]+)/datalayer/delete/(?P<pk>\d+)/$',
        views.DataLayerDelete.as_view(), name='datalayer_delete'),
)



import pytest

from leaflet_storage.models import DataLayer, Map

pytestmark = pytest.mark.django_db


def test_licence_delete_should_not_remove_linked_maps(map, licence, datalayer):
    assert map.licence == licence
    licence.delete()
    assert Map.objects.filter(pk=map.pk).exists()
    assert DataLayer.objects.filter(pk=datalayer.pk).exists()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs

from setuptools import setup, find_packages

import leaflet_storage

long_description = codecs.open('README.md', "r", "utf-8").read()

setup(
    name="django-leaflet-storage",
    version=leaflet_storage.__version__,
    author=leaflet_storage.__author__,
    author_email=leaflet_storage.__contact__,
    description=leaflet_storage.__doc__,
    keywords="django leaflet geodjango",
    url=leaflet_storage.__homepage__,
    download_url="https://github.com/yohanboniface/django-leaflet-storage/downloads",
    packages=find_packages(),
    include_package_data=True,
    platforms=["any"],
    zip_safe=False,
    long_description=long_description,

    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        #"License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
    ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.contrib.gis.db.models.fields
import leaflet_storage.models
import django.db.models.deletion
from django.conf import settings
import leaflet_storage.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DataLayer',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='name')),
                ('description', models.TextField(null=True, verbose_name='description', blank=True)),
                ('geojson', models.FileField(null=True, upload_to=leaflet_storage.models.upload_to, blank=True)),
                ('display_on_load', models.BooleanField(default=False, help_text='Display this layer on load.', verbose_name='display on load')),
            ],
            options={
                'ordering': ('name',),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Licence',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='name')),
                ('details', models.URLField(help_text='Link to a page where the licence is detailed.', verbose_name='details')),
            ],
            options={
                'ordering': ('name',),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Map',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='name')),
                ('slug', models.SlugField()),
                ('description', models.TextField(null=True, verbose_name='description', blank=True)),
                ('center', django.contrib.gis.db.models.fields.PointField(srid=4326, verbose_name='center', geography=True)),
                ('zoom', models.IntegerField(default=7, verbose_name='zoom')),
                ('locate', models.BooleanField(default=False, help_text='Locate user on load?', verbose_name='locate')),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('edit_status', models.SmallIntegerField(default=3, verbose_name='edit status', choices=[(1, 'Everyone can edit'), (2, 'Only editors can edit'), (3, 'Only owner can edit')])),
                ('share_status', models.SmallIntegerField(default=1, verbose_name='share status', choices=[(1, 'everyone (public)'), (2, 'anyone with link'), (3, 'editors only')])),
                ('settings', leaflet_storage.fields.DictField(null=True, verbose_name='settings', blank=True)),
                ('editors', models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name='editors', blank=True)),
                ('licence', models.ForeignKey(on_delete=django.db.models.deletion.SET_DEFAULT, default=leaflet_storage.models.get_default_licence, verbose_name='licence', to='leaflet_storage.Licence', help_text='Choose the map licence.')),
                ('owner', models.ForeignKey(related_name='owned_maps', verbose_name='owner', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'ordering': ('name',),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Pictogram',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='name')),
                ('attribution', models.CharField(max_length=300)),
                ('pictogram', models.ImageField(upload_to=b'pictogram')),
            ],
            options={
                'ordering': ('name',),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TileLayer',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200, verbose_name='name')),
                ('url_template', models.CharField(help_text='URL template using OSM tile format', max_length=200)),
                ('minZoom', models.IntegerField(default=0)),
                ('maxZoom', models.IntegerField(default=18)),
                ('attribution', models.CharField(max_length=300)),
                ('rank', models.SmallIntegerField(help_text='Order of the tilelayers in the edit box', null=True, blank=True)),
            ],
            options={
                'ordering': ('rank', 'name'),
            },
        ),
        migrations.AddField(
            model_name='map',
            name='tilelayer',
            field=models.ForeignKey(related_name='maps', verbose_name='background', blank=True, to='leaflet_storage.TileLayer', null=True),
        ),
        migrations.AddField(
            model_name='datalayer',
            name='map',
            field=models.ForeignKey(to='leaflet_storage.Map'),
        ),
    ]
